<?php
namespace FormStatus\App;

class FormStatus
{
    public function __construct()
    {
		add_action('init', array($this, 'start_session'));
        add_action('init', array($this, 'set_session_variables'));
		add_action('init', array($this, 'include_plugin_mods'));
		add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'));
		add_filter('formstatus_form_identifier', array($this, 'add_jetpack_identifier_support'), 10, 4);
		add_filter('render_block', array($this, 'add_form_identifier_in_block_content'), 99999, 2);
        add_filter('do_shortcode_tag', array($this, 'add_form_identifier_in_shortcode'), 99999, 4);
		add_filter('pre_wp_mail', array($this, 'disable_wp_mail'), 99999, 2);
        add_filter('wp_mail', array($this, 'override_wp_mail'), 99999);
        add_action('wp_mail_succeeded', array($this, 'prevent_wp_mail_send'), 10);
    }
	
	public function start_session()
    {
        if (session_status() !== PHP_SESSION_ACTIVE) {
            session_start();
        }
    }

    public function set_session_variables()
    {
        $_SESSION['formstatus'] = formstatus_parse_url_parameters();
        $_SESSION['formstatus_can_send_mail'] = 'yes';
    }
	
	public function include_plugin_mods()
    {
		if (!function_exists('is_plugin_active')) {
			require_once(ABSPATH . '/wp-admin/includes/plugin.php');
		}
		
        $plugins = array(
            'contact-form-7/wp-contact-form-7.php' => 'contact-form-7.php',
            'gravityforms/gravityforms.php' => 'gravity-forms.php',
			'wpforms/wpforms.php' => 'wp-forms.php',
			'wpforms-lite/wpforms.php' => 'wp-forms.php',
			'ninja-forms/ninja-forms.php' => 'ninja-forms.php',
        );

		if ($plugins) {
			foreach ($plugins as $plugin => $mod_file) {
				if (!is_plugin_active($plugin)) {
					continue;
				}
				require_once FORMSTATUS_PLUGIN_DIR . 'plugin-mods' . DIRECTORY_SEPARATOR . $mod_file;
			}
		}
    }

    public function enqueue_scripts()
    {
		if (!empty($_SESSION['formstatus'])) {
            wp_enqueue_script('formstatus-js', FORMSTATUS_PLUGIN_URL . 'assets/js/formstatus.js', array(), FORMSTATUS_VERSION, true);
        }
    }
	
	/**
     * Generates the form identifier based on the attribute ID
     *
     * @param $tag
     * @param $attr
     * @return string|null
     */
    private function generate_form_identifier($tag, $attr = array())
    {
        $form_id = false;

        if (empty($attr)) {
            return '';
        }

        foreach ($attr as $key => $value) {
            if (strtolower($key) == 'id') {
                $form_id = $value;
                break;
            }
			
            if (strtolower(substr($key, -3)) == '_id') {
                $form_id = $value;
                break;
            }
        }

        formstatus_write_log(['function' => 'FormStatus\App\FormStatus::generate_form_identifier', 'tag' => $tag, 'attr' => $attr, 'form_id' => $form_id]);

        if (!$form_id) {
            return null;
        }

        return md5($tag . $form_id);
    }
	
	/**
     * Use Jetpack form contact-form-hash field value as the identifier
     *
     * @param $identifier
     * @param $output
     * @param $tag
     * @param $attr
     * @return mixed|string
     */
    public function add_jetpack_identifier_support($identifier, $output, $tag, $attr)
    {
        if ($tag !== 'jetpack/contact-form' && $tag !== 'contact-form') {
            return $identifier;
        }

        if (preg_match('/<input\s+type=["\']hidden["\']\s+name=["\']contact-form-hash["\']\s+value=["\'](.*?)["\']/i', $output, $matches)) {
            formstatus_write_log(['function' => 'FormStatus\App\FormStatus::add_jetpack_identifier_support', 'form_hash' => $matches[1]]);

            return md5($matches[1]);
        }

        return $identifier;
    }
	
	/**
     * Adds an attribute formstatus as the form identifier.
     *
     * @param $output
     * @param $tag
     * @param $attr
     * @return array|mixed|string|string[]|null
     */
    private function get_form_output_with_id($output, $tag, $attr)
    {
        $pattern = '/<form(.*?)>/';

        // Only perform the actions below if the output contains a form tag.
        if (!preg_match($pattern, $output)) {
            return $output;
        }

        $formstatus_attr = '';
        $identifier = apply_filters(
            'formstatus_form_identifier',
            $this->generate_form_identifier($tag, $attr),
            $output,
            $tag,
            $attr
        );

        if (!empty($identifier)) {
            $formstatus_attr = 'formstatus="' . $identifier . '"';
        }

        $form_has_support = apply_filters('formstatus_support', false, $tag);
        $formstatus_support = ($form_has_support) ? ' formstatus-support="1" ' : '';

        $replacement = '<form$1 data-formstatus="1" ' . $formstatus_attr . $formstatus_support . '>';

        formstatus_write_log(['function' => 'FormStatus\App\FormStatus::get_form_output_with_id', 'tag' => $tag, 'attr' => $attr, 'identifier' => $identifier, 'has_support' => $form_has_support]);

        return preg_replace('/<form(.*?)>/', $replacement, $output, 1);
    }
	
	/**
     * Modify the gutenberg block output to add the attribute formstatus in the form tag.
     *
     * @param $output
     * @param $block
     * @return mixed|void
     */
    public function add_form_identifier_in_block_content($output, $block)
    {
        $output = $this->get_form_output_with_id($output, $block['blockName'], $block['attrs']);

        return apply_filters('formstatus_block_content_output', $output, $block['blockName'], $block['attrs']);
    }
	
	/**
     * Modify the shortcode output to add the attribute formstatus in the form tag.
     *
     * @param $output
     * @param $tag
     * @param $attr
     * @param $m
     * @return mixed|void
     */
    public function add_form_identifier_in_shortcode($output, $tag, $attr, $m)
    {
        $output = $this->get_form_output_with_id($output, $tag, $attr);

        return apply_filters('formstatus_shortcode_content_output', $output, $tag, $attr);
    }
	
	public function disable_wp_mail($is_null, $attr)
    {
        if (empty($_SESSION['formstatus'])) {
            return $is_null;
        }
		
		$formstatus_can_send_mail = isset($_SESSION['formstatus_can_send_mail']) ? sanitize_text_field($_SESSION['formstatus_can_send_mail']) : '';
		$formstatus_postmarkemail = isset($_SESSION['formstatus']['postmarkEmail']) ? sanitize_email($_SESSION['formstatus']['postmarkEmail']) : '';

        formstatus_write_log(['function' => 'FormStatus\App\FormStatus::disable_wp_mail', 'can_send_mail' => $formstatus_can_send_mail, 'postmark_email' => $formstatus_postmarkemail, 'default_return' => $is_null]);

        if ($formstatus_can_send_mail === 'no' || // disable succeeding emails for form test submission.
            empty($formstatus_postmarkemail)) {
            return true;
        }

        return $is_null;
    }
	
	function override_wp_mail($args)
    {
        if (empty($_SESSION['formstatus'])) {
            return $args;
        }
		
		$formstatus_formid = isset($_SESSION['formstatus']['formId']) ? sanitize_text_field($_SESSION['formstatus']['formId']) : '';
		$formstatus_postmarkemail = isset($_SESSION['formstatus']['postmarkEmail']) ? sanitize_email($_SESSION['formstatus']['postmarkEmail']) : '';
		$formstatus_hashid = isset($_SESSION['formstatus']['hashId']) ? sanitize_text_field($_SESSION['formstatus']['hashId']) : '';

        $email = str_replace('@', '+' . $formstatus_formid . '@', $formstatus_postmarkemail);

        $site_url = get_site_url();
        $domain = preg_replace('/^https?:\/\//', '', $site_url);
        $domain = rtrim($domain, '/');

        $args = array(
            'to' => $email,
            'subject' => 'formstatus-' . $formstatus_formid,
            'message' => '{{' . $formstatus_hashid . '}}',
            'headers' => array(
                'From' => '"no-reply@' . $domain . '" <no-reply@' . $domain . '>',
                'Content-type' => 'Content-type: text/html; charset=UTF-8'
            )
        );

        formstatus_write_log(['function' => 'FormStatus\App\FormStatus::override_wp_mail', 'form_id' => $formstatus_formid, 'postmark_email' => $formstatus_postmarkemail, 'hash_id' => $formstatus_hashid, 'domain' => $domain, 'email' => $email]);

        return $args;
    }
	
    public function prevent_wp_mail_send()
    {
		if (empty($_SESSION['formstatus'])) {
            return;
        }

        $_SESSION['formstatus_can_send_mail'] = 'no';

        formstatus_write_log(['function' => 'FormStatus\App\FormStatus::prevent_wp_mail_send', 'can_send_mail' => $_SESSION['formstatus_can_send_mail']]);
    }
} // end class